import 'dart:async';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import 'phone_verification_widget.dart';

const errorKeys = {
  "account-exists-with-different-credential":
      "Account exists with different credential",
  "invalid-credential": "Invalid credential",
  "operation-not-allowed": "Operation not allowed",
  "user-disabled": "User disabled",
  "user-not-found": "User not found",
  "wrong-password": "Wrong password",
  "app-not-authorized": "App not authorized",
  "invalid-verification-code": "Invalid verification code"
};

class PhoneAuthService {
  final FirebaseAuth _auth = FirebaseAuth.instance;
  Completer<PhoneAuthResponse> _verifyCompleter;
  bool _isVerificationShowing = false;

  GlobalKey<NavigatorState> get _navigatorKey {
    return Get.key;
  }

  BuildContext get context {
    return _navigatorKey.currentState.overlay.context;
  }

  int _forceResendingToken;
  String _verificationId;
  String _phoneNumber;

  // final _loadingService = locator<LoadingService>();

  /// Phone number in e164 format. Ex: +84933723831
  Future<PhoneAuthResponse> verifyPhoneNumber(String phoneNumber) async {
    await _clean();
    _verifyCompleter = Completer<PhoneAuthResponse>();
    _requestPhoneOTP(phoneNumber);
    return _verifyCompleter.future;
  }

  _requestPhoneOTP(String phoneNumber) async {
    _phoneNumber = phoneNumber;
    // await _loadingService.showLoading();
    await _auth.verifyPhoneNumber(
        phoneNumber: phoneNumber,
        timeout: const Duration(seconds: 60),
        verificationCompleted: (AuthCredential credential) async {
          // await _loadingService.hideLoading();
          _verifyAuthCredential(credential);
        },
        verificationFailed: (FirebaseAuthException authException) async {
          _verifyCompleter.completeError(authException.code);
          // await _loadingService.hideLoading();
          _navigatorKey.currentState.pop();
          await _clean();
        },
        codeSent: (String verificationId, int forceResendingToken) {
          _verificationId = verificationId;
          _forceResendingToken = _forceResendingToken;
          _showVerificationDialog();
        },
        forceResendingToken: _forceResendingToken,
        codeAutoRetrievalTimeout: (String verificationId) {
          _verificationId = verificationId;
        });
  }

  _verifyOtp(String otpCode) async {
    // await _loadingService.showLoading();
    final AuthCredential credential = PhoneAuthProvider.credential(
      verificationId: _verificationId,
      smsCode: otpCode,
    );
    _verifyAuthCredential(credential);
  }

  _verifyAuthCredential(AuthCredential credential) async {
    try {
      final _userCredential = await _auth.signInWithCredential(credential);
      final verifyToken = await _userCredential.user.getIdToken();
      _verifyCompleter.complete(PhoneAuthResponse(verifyToken));
      _navigatorKey.currentState.pop();
      // await _loadingService.hideLoading();
      await _clean();
    } catch (error) {
      // TODO: Show error popup
      // await _loadingService.hideLoading();
      // logger.e(error);
      // locator<DialogService>().showDialog(
      //     title: kAppName, description: allTranslations.text(error.code));
    }
  }

  _resendOtp() {
    _requestPhoneOTP(_phoneNumber);
  }

  _showVerificationDialog() async {
    // await _loadingService.hideLoading();
    if (_isVerificationShowing) {
      _navigatorKey.currentState.pop();
    }
    _isVerificationShowing = true;
    showModalBottomSheet(
      backgroundColor: Colors.transparent,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(
            topLeft: Radius.circular(20.0), topRight: Radius.circular(20.0)),
      ),
      isScrollControlled: true,
      context: context,
      builder: (_) {
        return PhoneVerificationWidget(
          onSubmit: (code) {
            _verifyOtp(code);
          },
          onResend: () {
            _resendOtp();
          },
          onDone: (code) {
            _verifyOtp(code);
          },
          pinLength: 6,
        );
      },
    ).whenComplete(() {
      _isVerificationShowing = false;
    });
  }

  _clean() async {
    await _auth.signOut();
    _verifyCompleter = null;
    _verificationId = null;
    _forceResendingToken = null;
    _phoneNumber = null;
  }
}

class PhoneAuthResponse {
  final String verifyToken;

  PhoneAuthResponse(this.verifyToken);
}
